@@ -2,7 +2,7 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2015 Alessandro Bono
  * Copyright © 2014 – 2015 Pranav Kant
- * Copyright © 2012 – 2016 Red Hat, Inc.
+ * Copyright © 2012 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,8 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+
 #include <gegl.h>
 #include <gexiv2/gexiv2.h>
 #include <gio/gio.h>
@@ -43,6 +45,7 @@
 #include "photos-export-dialog.h"
 #include "photos-export-notification.h"
 #include "photos-filterable.h"
+#include "photos-glib.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
 #include "photos-properties-dialog.h"
@@ -60,6 +63,7 @@
 #include "photos-share-dialog.h"
 #include "photos-share-notification.h"
 #include "photos-share-point-manager.h"
+#include "photos-thumbnail-factory.h"
 #include "photos-tracker-extract-priority.h"
 #include "photos-utils.h"
 
@@ -74,11 +78,11 @@ struct _PhotosApplication
   GResource *resource;
   GSettings *bg_settings;
   GSettings *ss_settings;
+  GSimpleAction *blacks_exposure_action;
   GSimpleAction *brightness_contrast_action;
   GSimpleAction *crop_action;
   GSimpleAction *delete_action;
   GSimpleAction *denoise_action;
-  GSimpleAction *draw_action;
   GSimpleAction *edit_action;
   GSimpleAction *edit_cancel_action;
   GSimpleAction *edit_done_action;
@@ -109,6 +113,7 @@ struct _PhotosApplication
   PhotosSearchContextState *state;
   PhotosSearchProvider *search_provider;
   PhotosSelectionController *sel_cntrlr;
+  PhotosThumbnailFactory *factory;
   TrackerExtractPriority *extract_priority;
   gboolean main_window_deleted;
   guint create_miners_count;
@@ -150,9 +155,11 @@ static const gchar *REQUIRED_GEGL_OPS[] =
   "gegl:buffer-sink",
   "gegl:buffer-source",
   "gegl:crop",
+  "gegl:exposure",
   "gegl:gray",
   "gegl:load",
   "gegl:nop",
+  "gegl:pixbuf",
   "gegl:rotate-on-center",
   "gegl:save-pixbuf",
   "gegl:scale-ratio",
@@ -323,6 +330,7 @@ photos_application_actions_update (PhotosApplication *self)
   selection_mode = photos_selection_controller_get_selection_mode (self->sel_cntrlr);
 
   enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
+  g_simple_action_set_enabled (self->blacks_exposure_action, enable);
   g_simple_action_set_enabled (self->brightness_contrast_action, enable);
   g_simple_action_set_enabled (self->crop_action, enable);
   g_simple_action_set_enabled (self->denoise_action, enable);
@@ -397,10 +405,6 @@ photos_application_actions_update (PhotosApplication *self)
             && mode == PHOTOS_WINDOW_MODE_PREVIEW
             && photos_base_item_can_edit (item));
   g_simple_action_set_enabled (self->edit_action, enable);
-
-  enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
-            || mode == PHOTOS_WINDOW_MODE_EDIT);
-  g_simple_action_set_enabled (self->draw_action, enable);
 }
 
 
@@ -624,6 +628,7 @@ photos_application_create_window (PhotosApplication *self)
 {
   gboolean gexiv2_initialized;
   gboolean gexiv2_registered_namespace;
+  gboolean sanity_checked_gegl;
 
   if (self->main_window != NULL)
     return TRUE;
@@ -634,7 +639,8 @@ photos_application_create_window (PhotosApplication *self)
   gexiv2_registered_namespace = gexiv2_metadata_register_xmp_namespace ("http://www.gnome.org/xmp", "gnome");
   g_return_val_if_fail (gexiv2_registered_namespace, FALSE);
 
-  g_return_val_if_fail (photos_application_sanity_check_gegl (self), FALSE);
+  sanity_checked_gegl = photos_application_sanity_check_gegl (self);
+  g_return_val_if_fail (sanity_checked_gegl, FALSE);
 
   self->main_window = photos_main_window_new (GTK_APPLICATION (self));
   g_signal_connect_object (self->main_window,
@@ -645,6 +651,7 @@ photos_application_create_window (PhotosApplication *self)
   g_signal_connect_swapped (self->main_window, "destroy", G_CALLBACK (photos_application_destroy), self);
 
   self->main_window_deleted = FALSE;
+  self->factory = photos_thumbnail_factory_dup_singleton (NULL, NULL);
 
   g_application_hold (G_APPLICATION (self));
   tracker_extract_priority_proxy_new_for_bus (G_BUS_TYPE_SESSION,
@@ -754,7 +761,7 @@ photos_application_edit_cancel_process (GObject *source_object, GAsyncResult *re
   GError *error = NULL;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
 
-  if (!photos_base_item_operations_revert_finish (item, res, &error))
+  if (!photos_base_item_pipeline_revert_finish (item, res, &error))
     {
       g_warning ("Unable to process item: %s", error->message);
       g_error_free (error);
@@ -779,7 +786,7 @@ photos_application_edit_cancel (PhotosApplication *self)
   g_return_if_fail (item != NULL);
 
   g_application_hold (G_APPLICATION (self));
-  photos_base_item_operations_revert_async (item, NULL, photos_application_edit_cancel_process, self);
+  photos_base_item_pipeline_revert_async (item, NULL, photos_application_edit_cancel_process, self);
 }
 
 
@@ -822,7 +829,7 @@ photos_application_edit_revert_revert (GObject *source_object, GAsyncResult *res
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
 
   error = NULL;
-  if (!photos_base_item_operations_revert_finish (item, res, &error))
+  if (!photos_base_item_pipeline_revert_finish (item, res, &error))
     {
       g_warning ("Unable to process item: %s", error->message);
       g_error_free (error);
@@ -833,7 +840,6 @@ photos_application_edit_revert_revert (GObject *source_object, GAsyncResult *res
   photos_base_item_pipeline_save_async (item, NULL, photos_application_edit_revert_pipeline_save, self);
 
  out:
-  g_action_activate (G_ACTION (self->draw_action), NULL);
   g_application_release (G_APPLICATION (self));
 }
 
@@ -851,7 +857,7 @@ photos_application_edit_revert (PhotosApplication *self, GVariant *parameter)
   g_return_if_fail (item != NULL);
 
   g_application_hold (G_APPLICATION (self));
-  photos_base_item_operations_revert_async (item, NULL, photos_application_edit_revert_revert, self);
+  photos_base_item_pipeline_revert_async (item, NULL, photos_application_edit_revert_revert, self);
 }
 
 
@@ -984,7 +990,6 @@ photos_application_properties_revert_to_original (GObject *source_object, GAsync
   photos_base_item_pipeline_save_async (item, NULL, photos_application_properties_pipeline_save, self);
 
  out:
-  g_action_activate (G_ACTION (self->draw_action), NULL);
   g_application_release (G_APPLICATION (self));
 }
 
@@ -1232,7 +1237,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   export = g_file_new_for_path (export_path);
 
   error = NULL;
-  if (!photos_utils_make_directory_with_parents (export, NULL, &error))
+  if (!photos_glib_make_directory_with_parents (export, NULL, &error))
     {
       g_warning ("Unable to create %s: %s", export_path, error->message);
       photos_export_notification_new_with_error (error);
@@ -1256,7 +1261,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   export = tmp;
 
   error = NULL;
-  if (!photos_utils_make_directory_with_parents (export, NULL, &error))
+  if (!photos_glib_make_directory_with_parents (export, NULL, &error))
     {
       g_warning ("Unable to create %s: %s", export_path, error->message);
       photos_export_notification_new_with_error (error);
@@ -1588,7 +1593,7 @@ photos_application_handle_local_options (GApplication *application, GVariantDict
   if (g_variant_dict_contains (options, "version"))
     {
       g_print ("%s %s\n", PACKAGE_TARNAME, PACKAGE_VERSION);
-      ret_val = 0;
+      ret_val = EXIT_SUCCESS;
     }
 
   return ret_val;
@@ -1695,6 +1700,11 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  parameter_type = g_variant_type_new ("a{sd}");
+  self->blacks_exposure_action = g_simple_action_new ("blacks-exposure-current", parameter_type);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->blacks_exposure_action));
+  g_variant_type_free (parameter_type);
+
   parameter_type = g_variant_type_new ("a{sd}");
   self->brightness_contrast_action = g_simple_action_new ("brightness-contrast-current", parameter_type);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->brightness_contrast_action));
@@ -1711,9 +1721,6 @@ photos_application_startup (GApplication *application)
   self->denoise_action = g_simple_action_new ("denoise-current", G_VARIANT_TYPE_UINT16);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->denoise_action));
 
-  self->draw_action = g_simple_action_new ("draw-current", NULL);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->draw_action));
-
   self->edit_cancel_action = g_simple_action_new ("edit-cancel", NULL);
   g_signal_connect_swapped (self->edit_cancel_action,
                             "activate",
@@ -1892,11 +1899,11 @@ photos_application_dispose (GObject *object)
   g_clear_object (&self->create_window_cancellable);
   g_clear_object (&self->bg_settings);
   g_clear_object (&self->ss_settings);
+  g_clear_object (&self->blacks_exposure_action);
   g_clear_object (&self->brightness_contrast_action);
   g_clear_object (&self->crop_action);
   g_clear_object (&self->delete_action);
   g_clear_object (&self->denoise_action);
-  g_clear_object (&self->draw_action);
   g_clear_object (&self->edit_action);
   g_clear_object (&self->edit_cancel_action);
   g_clear_object (&self->edit_done_action);
@@ -1924,6 +1931,7 @@ photos_application_dispose (GObject *object)
   g_clear_object (&self->shr_pnt_mngr);
   g_clear_object (&self->camera_cache);
   g_clear_object (&self->sel_cntrlr);
+  g_clear_object (&self->factory);
   g_clear_object (&self->extract_priority);
 
   if (self->state != NULL)
@@ -2007,7 +2015,7 @@ photos_application_search_context_iface_init (PhotosSearchContextInterface *ifac
 }
 
 
-GtkApplication *
+GApplication *
 photos_application_new (void)
 {
   return g_object_new (PHOTOS_TYPE_APPLICATION,
