@@ -1,7 +1,7 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2014 Pranav Kant
- * Copyright © 2012 – 2016 Red Hat, Inc.
+ * Copyright © 2012 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -142,6 +142,8 @@ const gchar        *photos_base_item_get_name                (PhotosBaseItem *se
 
 const gchar        *photos_base_item_get_name_with_fallback  (PhotosBaseItem *self);
 
+GQuark              photos_base_item_get_orientation         (PhotosBaseItem *self);
+
 GdkPixbuf          *photos_base_item_get_original_icon       (PhotosBaseItem *self);
 
 const gchar        *photos_base_item_get_resource_urn        (PhotosBaseItem *self);
@@ -225,15 +227,6 @@ gboolean            photos_base_item_operation_remove_finish (PhotosBaseItem *se
                                                               GAsyncResult *res,
                                                               GError **error);
 
-void                photos_base_item_operations_revert_async (PhotosBaseItem *self,
-                                                              GCancellable *cancellable,
-                                                              GAsyncReadyCallback callback,
-                                                              gpointer user_data);
-
-gboolean            photos_base_item_operations_revert_finish(PhotosBaseItem *self,
-                                                              GAsyncResult *res,
-                                                              GError **error);
-
 void                photos_base_item_pipeline_is_edited_async          (PhotosBaseItem *self,
                                                                         GCancellable *cancellable,
                                                                         GAsyncReadyCallback callback,
@@ -243,6 +236,15 @@ gboolean            photos_base_item_pipeline_is_edited_finish         (PhotosBa
                                                                         GAsyncResult *res,
                                                                         GError **error);
 
+void                photos_base_item_pipeline_revert_async             (PhotosBaseItem *self,
+                                                                        GCancellable *cancellable,
+                                                                        GAsyncReadyCallback callback,
+                                                                        gpointer user_data);
+
+gboolean            photos_base_item_pipeline_revert_finish            (PhotosBaseItem *self,
+                                                                        GAsyncResult *res,
+                                                                        GError **error);
+
 void                photos_base_item_pipeline_revert_to_original_async (PhotosBaseItem *self,
                                                                         GCancellable *cancellable,
                                                                         GAsyncReadyCallback callback,
