@@ -1,7 +1,7 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2013 Álvaro Peña
- * Copyright © 2014 – 2016 Red Hat, Inc.
+ * Copyright © 2014 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,10 +32,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <goa/goa.h>
-#include <libgnome-desktop/gnome-desktop-thumbnail.h>
 
 #include "photos-base-manager.h"
 #include "photos-debug.h"
+#include "photos-error.h"
 #include "photos-facebook-item.h"
 #include "photos-search-context.h"
 #include "photos-source.h"
@@ -138,14 +138,19 @@ photos_facebook_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cance
   GFBGraphPhoto *photo = NULL;
   const GFBGraphPhotoImage *thumbnail_image;
   gboolean ret_val = FALSE;
+  const gchar *uri;
   gchar *local_dir = NULL;
   gchar *local_path = NULL;
+  gint64 height;
+  gint64 width;
+  guint size;
 
   photo = photos_facebook_get_gfbgraph_photo (item, cancellable, error);
   if (photo == NULL)
     goto out;
 
-  thumbnail_image = gfbgraph_photo_get_image_near_width (photo, photos_utils_get_icon_size ());
+  size = (guint) photos_utils_get_icon_size ();
+  thumbnail_image = gfbgraph_photo_get_image_near_width (photo, size);
   if (thumbnail_image == NULL)
     {
       g_set_error (error, PHOTOS_ERROR, 0, "Failed to find an image for the thumbnail");
@@ -154,20 +159,24 @@ photos_facebook_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cance
 
   remote_file = g_file_new_for_uri (thumbnail_image->source);
 
-  local_path = gnome_desktop_thumbnail_path_for_uri (photos_base_item_get_uri (item),
-                                                     GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
+  uri = photos_base_item_get_uri (item);
+  local_path = photos_utils_get_thumbnail_path_for_uri (uri);
   local_file = g_file_new_for_path (local_path);
+
   local_dir = g_path_get_dirname (local_path);
   g_mkdir_with_parents (local_dir, 0700);
 
+  height = photos_base_item_get_height (item);
+  width = photos_base_item_get_width (item);
+
   photos_debug (PHOTOS_DEBUG_NETWORK, "Downloading %s from Facebook to %s", thumbnail_image->source, local_path);
-  if (!g_file_copy (remote_file,
-                    local_file,
-                    G_FILE_COPY_ALL_METADATA | G_FILE_COPY_OVERWRITE,
-                    cancellable,
-                    NULL,
-                    NULL,
-                    error))
+  if (!photos_utils_file_copy_as_thumbnail (remote_file,
+                                            local_file,
+                                            uri,
+                                            height,
+                                            width,
+                                            cancellable,
+                                            error))
     goto out;
 
   ret_val = TRUE;
