@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2013 – 2016 Red Hat, Inc.
+ * Copyright © 2013 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,10 +32,10 @@
 #include <glib/gi18n.h>
 #include <goa/goa.h>
 #include <grilo.h>
-#include <libgnome-desktop/gnome-desktop-thumbnail.h>
 
 #include "photos-base-manager.h"
 #include "photos-debug.h"
+#include "photos-error.h"
 #include "photos-flickr-item.h"
 #include "photos-search-context.h"
 #include "photos-source.h"
@@ -168,6 +168,8 @@ photos_flickr_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancell
   gchar *grilo_id = NULL;
   gchar *local_dir = NULL;
   gchar *local_path = NULL;
+  gint64 height;
+  gint64 width;
   gsize prefix_len;
 
   data.error = error;
@@ -233,20 +235,23 @@ photos_flickr_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancell
   remote_file = g_file_new_for_uri (thumbnail_uri);
 
   uri = photos_base_item_get_uri (item);
-  local_path = gnome_desktop_thumbnail_path_for_uri (uri, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  local_path = photos_utils_get_thumbnail_path_for_uri (uri);
   local_file = g_file_new_for_path (local_path);
 
   local_dir = g_path_get_dirname (local_path);
   g_mkdir_with_parents (local_dir, 0700);
 
+  height = photos_base_item_get_height (item);
+  width = photos_base_item_get_width (item);
+
   photos_debug (PHOTOS_DEBUG_NETWORK, "Downloading %s from Flickr to %s", thumbnail_uri, local_path);
-  if (!g_file_copy (remote_file,
-                    local_file,
-                    G_FILE_COPY_ALL_METADATA | G_FILE_COPY_OVERWRITE,
-                    cancellable,
-                    NULL,
-                    NULL,
-                    error))
+  if (!photos_utils_file_copy_as_thumbnail (remote_file,
+                                            local_file,
+                                            uri,
+                                            height,
+                                            width,
+                                            cancellable,
+                                            error))
     goto out;
 
   ret_val = TRUE;
