@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 – 2016 Red Hat, Inc.
+ * Copyright © 2012 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,11 +50,6 @@ struct _PhotosIndexingNotification
   guint timeout_id;
 };
 
-struct _PhotosIndexingNotificationClass
-{
-  GtkGridClass parent_class;
-};
-
 
 G_DEFINE_TYPE (PhotosIndexingNotification, photos_indexing_notification, GTK_TYPE_GRID);
 
@@ -290,6 +285,8 @@ photos_indexing_notification_init (PhotosIndexingNotification *self)
 
   close = gtk_button_new ();
   gtk_widget_set_valign (close, GTK_ALIGN_CENTER);
+  gtk_widget_set_focus_on_click (close, FALSE);
+  gtk_button_set_relief (GTK_BUTTON (close), GTK_RELIEF_NONE);
   gtk_button_set_image (GTK_BUTTON (close), image);
   gtk_container_add (GTK_CONTAINER (self), close);
   g_signal_connect_swapped (close, "clicked", G_CALLBACK (photos_indexing_notification_close_clicked), self);
