@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 – 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2017 Red Hat, Inc.
  * Copyright © 2016 Umang Jain
  *
  * This program is free software; you can redistribute it and/or
@@ -28,9 +28,9 @@
 
 #include "egg-counter.h"
 #include "photos-debug.h"
+#include "photos-gegl.h"
 #include "photos-operation-insta-common.h"
 #include "photos-pipeline.h"
-#include "photos-utils.h"
 
 
 struct _PhotosPipeline
@@ -66,6 +66,7 @@ EGG_DEFINE_COUNTER (instances, "PhotosPipeline", "Instances", "Number of PhotosP
 static const gchar *OPERATIONS[] =
 {
   "gegl:crop",
+  "gegl:noise-reduction",
   "photos:insta-filter"
 };
 
@@ -150,7 +151,7 @@ photos_pipeline_create_graph_from_xml (PhotosPipeline *self, const gchar *conten
     goto out;
 
   g_hash_table_remove_all (self->hash);
-  photos_utils_remove_children_from_node (self->graph);
+  photos_gegl_remove_children_from_node (self->graph);
 
   input = gegl_node_get_input_proxy (self->graph, "input");
   output = gegl_node_get_output_proxy (self->graph, "output");
@@ -450,7 +451,10 @@ photos_pipeline_new_finish (GAsyncResult *res, GError **error)
 
 
 void
-photos_pipeline_add (PhotosPipeline *self, const gchar *operation, const gchar *first_property_name, va_list ap)
+photos_pipeline_add_valist (PhotosPipeline *self,
+                            const gchar *operation,
+                            const gchar *first_property_name,
+                            va_list ap)
 {
   GeglNode *input;
   GeglNode *last;
@@ -488,7 +492,26 @@ photos_pipeline_add (PhotosPipeline *self, const gchar *operation, const gchar *
 
 
 gboolean
-photos_pipeline_get (PhotosPipeline *self, const gchar *operation, const gchar *first_property_name, va_list ap)
+photos_pipeline_get (PhotosPipeline *self, const gchar *operation, const gchar *first_property_name, ...)
+{
+  gboolean ret_val;
+  va_list ap;
+
+  g_return_val_if_fail (PHOTOS_IS_PIPELINE (self), FALSE);
+
+  va_start (ap, first_property_name);
+  ret_val = photos_pipeline_get_valist (self, operation, first_property_name, ap);
+  va_end (ap);
+
+  return ret_val;
+}
+
+
+gboolean
+photos_pipeline_get_valist (PhotosPipeline *self,
+                            const gchar *operation,
+                            const gchar *first_property_name,
+                            va_list ap)
 {
   GeglNode *node;
   gboolean ret_val = FALSE;
