@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 – 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2017 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,12 +52,17 @@ void                   photos_pipeline_new_async         (GeglNode *parent,
 PhotosPipeline        *photos_pipeline_new_finish        (GAsyncResult *res,
                                                           GError **error);
 
-void                   photos_pipeline_add               (PhotosPipeline *self,
+void                   photos_pipeline_add_valist        (PhotosPipeline *self,
                                                           const gchar *operation,
                                                           const gchar *first_property_name,
                                                           va_list ap);
 
 gboolean               photos_pipeline_get               (PhotosPipeline *self,
+                                                          const gchar *operation,
+                                                          const gchar *first_property_name,
+                                                          ...) G_GNUC_NULL_TERMINATED G_GNUC_WARN_UNUSED_RESULT;
+
+gboolean               photos_pipeline_get_valist        (PhotosPipeline *self,
                                                           const gchar *operation,
                                                           const gchar *first_property_name,
                                                           va_list ap) G_GNUC_WARN_UNUSED_RESULT;
