@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2015 – 2016 Red Hat, Inc.
+ * Copyright © 2015 – 2017 Red Hat, Inc.
  * Copyright © 2015 – 2016 Umang Jain
  * Copyright © 2011 – 2015 Yorba Foundation
  *
@@ -111,14 +111,6 @@ struct _PhotosToolCropConstraint
   guint basis_width;
 };
 
-enum
-{
-  CONSTRAINT_COLUMN_ASPECT_RATIO = 0,
-  CONSTRAINT_COLUMN_NAME = 1,
-  CONSTRAINT_COLUMN_BASIS_HEIGHT = 2,
-  CONSTRAINT_COLUMN_BASIS_WIDTH = 3
-};
-
 /* "Free" is excluded from the GtkListBox and represented by the
  *  GtkCheckButton. Adjust accordingly.
  */
@@ -912,6 +904,7 @@ photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer us
   photos_tool_crop_set_active (self, (gint) active);
 
   photos_tool_crop_surface_draw (self);
+  gtk_widget_queue_draw (self->view);
 
   self->size_allocate_id = g_signal_connect_object (self->view,
                                                     "size-allocate",
@@ -930,6 +923,7 @@ photos_tool_crop_reset_clicked (PhotosToolCrop *self)
 {
   self->reset = TRUE;
   photos_tool_crop_set_active (self, -1);
+  gtk_widget_queue_draw (self->view);
   g_signal_emit_by_name (self, "hide-requested");
 }
 
@@ -981,6 +975,7 @@ photos_tool_crop_activate (PhotosTool *tool, PhotosBaseItem *item, PhotosImageVi
     {
       photos_tool_crop_surface_create (self);
       photos_tool_crop_init_crop (self);
+      gtk_widget_queue_draw (self->view);
 
       self->size_allocate_id = g_signal_connect_object (self->view,
                                                         "size-allocate",
