@@ -32,13 +32,14 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome-desktop/gnome-desktop-thumbnail.h>
 #include <tracker-sparql.h>
 #include <libgd/gd.h>
 
 #include "photos-application.h"
+#include "photos-error.h"
 #include "photos-facebook-item.h"
 #include "photos-flickr-item.h"
+#include "photos-gegl.h"
 #include "photos-google-item.h"
 #include "photos-local-item.h"
 #include "photos-media-server-item.h"
@@ -46,20 +47,13 @@
 #include "photos-offset-favorites-controller.h"
 #include "photos-offset-overview-controller.h"
 #include "photos-offset-search-controller.h"
-#include "photos-operation-insta-curve.h"
-#include "photos-operation-insta-filter.h"
-#include "photos-operation-insta-hefe.h"
-#include "photos-operation-insta-hefe-curve.h"
-#include "photos-operation-insta-hefe-vignette.h"
-#include "photos-operation-jpg-guess-sizes.h"
-#include "photos-operation-png-guess-sizes.h"
-#include "photos-operation-saturation.h"
 #include "photos-query.h"
 #include "photos-share-point.h"
 #include "photos-share-point-email.h"
 #include "photos-share-point-google.h"
 #include "photos-share-point-online.h"
 #include "photos-source.h"
+#include "photos-thumbnail-factory.h"
 #include "photos-tool.h"
 #include "photos-tool-colors.h"
 #include "photos-tool-crop.h"
@@ -73,40 +67,20 @@
 #include "photos-utils.h"
 
 
-static const gdouble EPSILON = 1e-5;
+typedef struct _PhotosUtilsFileQueryInfoData PhotosUtilsFileQueryInfoData;
 
-typedef struct _PhotosUtilsFileCreateData PhotosUtilsFileCreateData;
-
-struct _PhotosUtilsFileCreateData
+struct _PhotosUtilsFileQueryInfoData
 {
-  GFile *dir;
-  GFileCreateFlags flags;
-  gchar *basename;
-  gchar *extension;
-  gint io_priority;
-  guint count;
+  GFileQueryInfoFlags flags;
+  gchar *attributes;
 };
 
+static const gdouble EPSILON = 1e-5;
 
-gboolean
-photos_utils_app_info_launch_uri (GAppInfo *appinfo,
-                                  const gchar *uri,
-                                  GAppLaunchContext *launch_context,
-                                  GError **error)
+enum
 {
-  GList *uris = NULL;
-  gboolean ret_val;
-
-  g_return_val_if_fail (G_IS_APP_INFO (appinfo), FALSE);
-  g_return_val_if_fail (uri != NULL && uri[0] != '\0', FALSE);
-  g_return_val_if_fail (launch_context == NULL || G_IS_APP_LAUNCH_CONTEXT (launch_context), FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  uris = g_list_prepend (uris, g_strdup (uri));
-  ret_val = g_app_info_launch_uris (appinfo, uris, launch_context, error);
-  g_list_free_full (uris, g_free);
-  return ret_val;
-}
+  THUMBNAIL_GENERATION = 0
+};
 
 
 static void
@@ -153,103 +127,6 @@ photos_utils_border_pixbuf (GdkPixbuf *pixbuf)
 }
 
 
-static GeglBuffer *
-photos_utils_buffer_zoom (GeglBuffer *buffer, gdouble zoom, GCancellable *cancellable, GError **error)
-{
-  GeglBuffer *ret_val = NULL;
-  GeglNode *buffer_sink;
-  GeglNode *buffer_source;
-  GeglNode *graph;
-  GeglNode *scale;
-
-  graph = gegl_node_new ();
-  buffer_source = gegl_node_new_child (graph, "operation", "gegl:buffer-source", "buffer", buffer, NULL);
-  scale = gegl_node_new_child (graph, "operation", "gegl:scale-ratio", "x", zoom, "y", zoom, NULL);
-  buffer_sink = gegl_node_new_child (graph, "operation", "gegl:buffer-sink", "buffer", &ret_val, NULL);
-  gegl_node_link_many (buffer_source, scale, buffer_sink, NULL);
-  gegl_node_process (buffer_sink);
-
-  g_object_unref (graph);
-  return ret_val;
-}
-
-
-static void
-photos_utils_buffer_zoom_in_thread_func (GTask *task,
-                                         gpointer source_object,
-                                         gpointer task_data,
-                                         GCancellable *cancellable)
-{
-  GeglBuffer *buffer = GEGL_BUFFER (source_object);
-  GeglBuffer *result = NULL;
-  GError *error;
-  const gchar *zoom_str = (const gchar *) task_data;
-  gchar *endptr;
-  gdouble zoom;
-
-  zoom = g_ascii_strtod (zoom_str, &endptr);
-  g_assert (*endptr == '\0');
-
-  error = NULL;
-  result = photos_utils_buffer_zoom (buffer, zoom, cancellable, &error);
-  if (error != NULL)
-    {
-      g_task_return_error (task, error);
-      goto out;
-    }
-
-  g_task_return_pointer (task, g_object_ref (result), g_object_unref);
-
- out:
-  g_clear_object (&result);
-}
-
-
-void
-photos_utils_buffer_zoom_async (GeglBuffer *buffer,
-                                gdouble zoom,
-                                GCancellable *cancellable,
-                                GAsyncReadyCallback callback,
-                                gpointer user_data)
-{
-  GTask *task;
-  gchar zoom_str[G_ASCII_DTOSTR_BUF_SIZE];
-
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
-  g_return_if_fail (zoom > 0.0);
-
-  task = g_task_new (buffer, cancellable, callback, user_data);
-  g_task_set_source_tag (task, photos_utils_buffer_zoom_async);
-
-  if (GEGL_FLOAT_EQUAL ((gfloat) zoom, 1.0))
-    {
-      g_task_return_pointer (task, g_object_ref (buffer), g_object_unref);
-      goto out;
-    }
-
-  g_ascii_dtostr (zoom_str, G_N_ELEMENTS (zoom_str), zoom);
-  g_task_set_task_data (task, g_strdup (zoom_str), g_free);
-
-  g_task_run_in_thread (task, photos_utils_buffer_zoom_in_thread_func);
-
- out:
-  g_object_unref (task);
-}
-
-
-GeglBuffer *
-photos_utils_buffer_zoom_finish (GeglBuffer *buffer, GAsyncResult *res, GError **error)
-{
-  GTask *task = G_TASK (res);
-
-  g_return_val_if_fail (g_task_is_valid (res, buffer), NULL);
-  g_return_val_if_fail (g_task_get_source_tag (task) == photos_utils_buffer_zoom_async, NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-  return g_task_propagate_pointer (task, error);
-}
-
-
 GdkPixbuf *
 photos_utils_center_pixbuf (GdkPixbuf *pixbuf, gint size)
 {
@@ -285,26 +162,6 @@ photos_utils_convert_path_to_uri (const gchar *path)
 }
 
 
-GeglBuffer *
-photos_utils_create_buffer_from_node (GeglNode *node)
-{
-  GeglBuffer *buffer = NULL;
-  GeglNode *buffer_sink;
-  GeglNode *graph;
-
-  graph = gegl_node_get_parent (node);
-  buffer_sink = gegl_node_new_child (graph,
-                                     "operation", "gegl:buffer-sink",
-                                     "buffer", &buffer,
-                                     NULL);
-  gegl_node_link (node, buffer_sink);
-  gegl_node_process (buffer_sink);
-  g_object_unref (buffer_sink);
-
-  return buffer;
-}
-
-
 GIcon *
 photos_utils_create_collection_icon (gint base_size, GList *pixbufs)
 {
@@ -406,33 +263,6 @@ photos_utils_create_collection_icon (gint base_size, GList *pixbufs)
 }
 
 
-GeglNode *
-photos_utils_create_orientation_node (GeglNode *parent, GQuark orientation)
-{
-  GeglNode *ret_val = NULL;
-  double degrees = 1.0;
-
-  if (orientation == PHOTOS_ORIENTATION_TOP)
-    goto out;
-
-  if (orientation == PHOTOS_ORIENTATION_BOTTOM)
-    degrees = -180.0;
-  else if (orientation == PHOTOS_ORIENTATION_LEFT)
-    degrees = -270.0;
-  else if (orientation == PHOTOS_ORIENTATION_RIGHT)
-    degrees = -90.0;
-
-  if (degrees < 0.0)
-    ret_val = gegl_node_new_child (parent, "operation", "gegl:rotate-on-center", "degrees", degrees, NULL);
-
- out:
-  if (ret_val == NULL)
-    ret_val = gegl_node_new_child (parent, "operation", "gegl:nop", NULL);
-
-  return ret_val;
-}
-
-
 GdkPixbuf *
 photos_utils_create_placeholder_icon_for_scale (const gchar *name, gint size, gint scale)
 {
@@ -490,26 +320,6 @@ photos_utils_create_placeholder_icon_for_scale (const gchar *name, gint size, gi
 }
 
 
-GdkPixbuf *
-photos_utils_create_pixbuf_from_node (GeglNode *node)
-{
-  GdkPixbuf *pixbuf = NULL;
-  GeglNode *graph;
-  GeglNode *save_pixbuf;
-
-  graph = gegl_node_get_parent (node);
-  save_pixbuf = gegl_node_new_child (graph,
-                                     "operation", "gegl:save-pixbuf",
-                                     "pixbuf", &pixbuf,
-                                     NULL);
-  gegl_node_link_many (node, save_pixbuf, NULL);
-  gegl_node_process (save_pixbuf);
-  g_object_unref (save_pixbuf);
-
-  return pixbuf;
-}
-
-
 GIcon *
 photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale)
 {
@@ -589,34 +399,32 @@ gboolean
 photos_utils_create_thumbnail (GFile *file,
                                const gchar *mime_type,
                                gint64 mtime,
+                               GQuark orientation,
+                               gint64 original_height,
+                               gint64 original_width,
+                               const gchar *pipeline_uri,
                                GCancellable *cancellable,
                                GError **error)
 {
-  GnomeDesktopThumbnailFactory *factory = NULL;
+  PhotosThumbnailFactory *factory = NULL;
   gboolean ret_val = FALSE;
-  gchar *uri = NULL;
-  GdkPixbuf *pixbuf = NULL;
 
-  uri = g_file_get_uri (file);
-  factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
-  pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (factory, uri, mime_type);
-  if (pixbuf == NULL)
-    {
-      /* FIXME: use proper #defines and enumerated types */
-      g_set_error (error,
-                   g_quark_from_static_string ("gnome-desktop-error"),
-                   0,
-                   "GnomeDesktopThumbnailFactory failed");
-      goto out;
-    }
+  factory = photos_thumbnail_factory_dup_singleton (NULL, NULL);
+  if (!photos_thumbnail_factory_generate_thumbnail (factory,
+                                                    file,
+                                                    mime_type,
+                                                    orientation,
+                                                    original_height,
+                                                    original_width,
+                                                    pipeline_uri,
+                                                    cancellable,
+                                                    error))
+    goto out;
 
-  gnome_desktop_thumbnail_factory_save_thumbnail (factory, pixbuf, uri, (time_t) mtime);
   ret_val = TRUE;
 
  out:
-  g_clear_object (&pixbuf);
   g_clear_object (&factory);
-  g_free (uri);
   return ret_val;
 }
 
@@ -634,7 +442,11 @@ photos_utils_get_thumbnail_icon (const gchar *uri)
   file = g_file_new_for_uri (uri);
 
   error = NULL;
-  info = g_file_query_info (file, G_FILE_ATTRIBUTE_THUMBNAIL_PATH, G_FILE_QUERY_INFO_NONE, NULL, &error);
+  info = photos_utils_file_query_info (file,
+                                       G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
+                                       G_FILE_QUERY_INFO_NONE,
+                                       NULL,
+                                       &error);
   if (error != NULL)
     {
       g_warning ("Unable to fetch thumbnail path for %s: %s", uri, error->message);
@@ -839,6 +651,7 @@ photos_utils_ensure_builtins (void)
   static gsize once_init_value = 0;
 
   photos_utils_ensure_extension_points ();
+  photos_gegl_ensure_builtins ();
 
   if (g_once_init_enter (&once_init_value))
     {
@@ -848,15 +661,6 @@ photos_utils_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_LOCAL_ITEM);
       g_type_ensure (PHOTOS_TYPE_MEDIA_SERVER_ITEM);
 
-      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_CURVE);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_FILTER);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_HEFE);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_HEFE_CURVE);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_HEFE_VIGNETTE);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_JPG_GUESS_SIZES);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_PNG_GUESS_SIZES);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_SATURATION);
-
       g_type_ensure (PHOTOS_TYPE_SHARE_POINT_EMAIL);
       g_type_ensure (PHOTOS_TYPE_SHARE_POINT_GOOGLE);
 
@@ -912,13 +716,6 @@ photos_utils_equal_double (gdouble a, gdouble b)
 }
 
 
-GQuark
-photos_utils_error_quark (void)
-{
-  return g_quark_from_static_string ("gnome-photos-error-quark");
-}
-
-
 gdouble
 photos_utils_eval_radial_line (gdouble crop_center_x,
                                gdouble crop_center_y,
@@ -938,6 +735,218 @@ photos_utils_eval_radial_line (gdouble crop_center_x,
 }
 
 
+gboolean
+photos_utils_file_copy_as_thumbnail (GFile *source,
+                                     GFile *destination,
+                                     const gchar *original_uri,
+                                     gint64 original_height,
+                                     gint64 original_width,
+                                     GCancellable *cancellable,
+                                     GError **error)
+{
+  GFileInputStream *istream = NULL;
+  GFileOutputStream *ostream = NULL;
+  GdkPixbuf *pixbuf = NULL;
+  gboolean ret_val = FALSE;
+  const gchar *prgname;
+  gchar *original_height_str = NULL;
+  gchar *original_width_str = NULL;
+
+  g_return_val_if_fail (G_IS_FILE (source), FALSE);
+  g_return_val_if_fail (G_IS_FILE (destination), FALSE);
+  g_return_val_if_fail (original_uri != NULL && original_uri[0] != '\0', FALSE);
+  g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  istream = g_file_read (source, cancellable, error);
+  if (istream == NULL)
+    goto out;
+
+  pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (istream), cancellable, error);
+  if (pixbuf == NULL)
+    goto out;
+
+  ostream = g_file_replace (destination,
+                            NULL,
+                            FALSE,
+                            G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION,
+                            cancellable,
+                            error);
+  if (ostream == NULL)
+    goto out;
+
+  original_height_str = g_strdup_printf ("%" G_GINT64_FORMAT, original_height);
+  original_width_str = g_strdup_printf ("%" G_GINT64_FORMAT, original_width);
+  prgname = g_get_prgname ();
+  if (!gdk_pixbuf_save_to_stream (pixbuf,
+                                  G_OUTPUT_STREAM (ostream),
+                                  "png",
+                                  cancellable,
+                                  error,
+                                  "tEXt::Software", prgname,
+                                  "tEXt::Thumb::URI", original_uri,
+                                  "tEXt::Thumb::Image::Height", original_height_str,
+                                  "tEXt::Thumb::Image::Width", original_width_str,
+                                  NULL))
+    {
+      goto out;
+    }
+
+  ret_val = TRUE;
+
+ out:
+  g_free (original_height_str);
+  g_free (original_width_str);
+  g_clear_object (&istream);
+  g_clear_object (&ostream);
+  g_clear_object (&pixbuf);
+  return ret_val;
+}
+
+
+GFileInfo *
+photos_utils_file_query_info (GFile *file,
+                              const gchar *attributes,
+                              GFileQueryInfoFlags flags,
+                              GCancellable *cancellable,
+                              GError **error)
+{
+  GFileAttributeMatcher *matcher = NULL;
+  GFileInfo *info = NULL;
+  GFileInfo *ret_val = NULL;
+
+  g_return_val_if_fail (G_IS_FILE (file), NULL);
+  g_return_val_if_fail (attributes != NULL && attributes[0] != '\0', NULL);
+  g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  info = g_file_query_info (file, attributes, flags, cancellable, error);
+  if (info == NULL)
+    goto out;
+
+  matcher = g_file_attribute_matcher_new (attributes);
+  if (g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID)
+      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAIL_PATH)
+      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED))
+    {
+      gchar *path = NULL;
+
+      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID);
+      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
+      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED);
+
+      path = photos_utils_get_thumbnail_path_for_file (file);
+      if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
+        {
+          g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID, TRUE);
+          g_file_info_set_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH, path);
+          g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED, FALSE);
+        }
+
+      g_free (path);
+    }
+
+  ret_val = g_object_ref (info);
+
+ out:
+  g_clear_object (&info);
+  g_clear_pointer (&matcher, (GDestroyNotify) g_file_attribute_matcher_unref);
+  return ret_val;
+}
+
+
+static void
+photos_utils_file_query_info_data_free (PhotosUtilsFileQueryInfoData *data)
+{
+  g_free (data->attributes);
+  g_slice_free (PhotosUtilsFileQueryInfoData, data);
+}
+
+
+static PhotosUtilsFileQueryInfoData *
+photos_utils_file_query_info_data_new (const gchar *attributes, GFileQueryInfoFlags flags)
+{
+  PhotosUtilsFileQueryInfoData *data;
+
+  data = g_slice_new0 (PhotosUtilsFileQueryInfoData);
+  data->flags = flags;
+  data->attributes = g_strdup (attributes);
+
+  return data;
+}
+
+
+static void
+photos_utils_file_query_info_in_thread_func (GTask *task,
+                                             gpointer source_object,
+                                             gpointer task_data,
+                                             GCancellable *cancellable)
+{
+  GError *error;
+  GFile *file = G_FILE (source_object);
+  GFileInfo *info = NULL;
+  PhotosUtilsFileQueryInfoData *data = (PhotosUtilsFileQueryInfoData *) task_data;
+
+  error = NULL;
+  info = photos_utils_file_query_info (file, data->attributes, data->flags, cancellable, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_pointer (task, g_object_ref (info), g_object_unref);
+
+ out:
+  g_clear_object (&info);
+}
+
+
+void
+photos_utils_file_query_info_async (GFile *file,
+                                    const gchar *attributes,
+                                    GFileQueryInfoFlags flags,
+                                    gint io_priority,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
+{
+  GTask *task;
+  PhotosUtilsFileQueryInfoData *data;
+  const gchar *wildcard;
+
+  g_return_if_fail (G_IS_FILE (file));
+  g_return_if_fail (attributes != NULL && attributes[0] != '\0');
+  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+  wildcard = strstr (attributes, "*");
+  g_return_if_fail (wildcard == NULL);
+
+  data = photos_utils_file_query_info_data_new (attributes, flags);
+
+  task = g_task_new (file, cancellable, callback, user_data);
+  g_task_set_priority (task, io_priority);
+  g_task_set_source_tag (task, photos_utils_file_query_info_async);
+  g_task_set_task_data (task, data, (GDestroyNotify) photos_utils_file_query_info_data_free);
+
+  g_task_run_in_thread (task, photos_utils_file_query_info_in_thread_func);
+  g_object_unref (task);
+}
+
+
+GFileInfo *
+photos_utils_file_query_info_finish (GFile *file, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, file), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_utils_file_query_info_async, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  return g_task_propagate_pointer (task, error);
+}
+
+
 void
 photos_utils_get_controller (PhotosWindowMode mode,
                              PhotosOffsetController **out_offset_cntrlr,
@@ -991,229 +1000,6 @@ photos_utils_get_controller (PhotosWindowMode mode,
 }
 
 
-static gchar *
-photos_utils_filename_get_extension_offset (const gchar *filename)
-{
-  gchar *end;
-  gchar *end2;
-
-  end = strrchr (filename, '.');
-
-  if (end != NULL && end != filename)
-    {
-      if (g_strcmp0 (end, ".gz") == 0
-          || g_strcmp0 (end, ".bz2") == 0
-          || g_strcmp0 (end, ".sit") == 0
-          || g_strcmp0 (end, ".Z") == 0)
-        {
-          end2 = end - 1;
-          while (end2 > filename && *end2 != '.')
-            end2--;
-          if (end2 != filename)
-            end = end2;
-        }
-  }
-
-  return end;
-}
-
-
-gchar *
-photos_utils_filename_strip_extension (const gchar *filename_with_extension)
-{
-  gchar *end;
-  gchar *filename;
-
-  if (filename_with_extension == NULL)
-    return NULL;
-
-  filename = g_strdup (filename_with_extension);
-  end = photos_utils_filename_get_extension_offset (filename);
-
-  if (end != NULL && end != filename)
-    *end = '\0';
-
-  return filename;
-}
-
-
-static void
-photos_utils_file_create_data_free (PhotosUtilsFileCreateData *data)
-{
-  g_object_unref (data->dir);
-  g_free (data->basename);
-  g_free (data->extension);
-  g_slice_free (PhotosUtilsFileCreateData, data);
-}
-
-
-static gchar *
-photos_utils_file_create_data_get_filename (PhotosUtilsFileCreateData *data)
-{
-  gchar *ret_val;
-
-  if (data->count > 0)
-    ret_val = g_strdup_printf ("%s(%u)%s", data->basename, data->count, data->extension);
-  else
-    ret_val = g_strdup_printf ("%s%s", data->basename, data->extension);
-
-  return ret_val;
-}
-
-
-static PhotosUtilsFileCreateData *
-photos_utils_file_create_data_new (GFile *file, GFileCreateFlags flags, gint io_priority)
-{
-  PhotosUtilsFileCreateData *data;
-  gchar *filename;
-
-  data = g_slice_new0 (PhotosUtilsFileCreateData);
-
-  filename = g_file_get_basename (file);
-  data->dir = g_file_get_parent (file);
-  data->basename = photos_utils_filename_strip_extension (filename);
-  data->extension = g_strdup (photos_utils_filename_get_extension_offset (filename));
-  data->count = 0;
-  data->flags = flags;
-  data->io_priority = io_priority;
-
-  g_free (filename);
-
-  return data;
-}
-
-
-static void
-photos_utils_file_create_create (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GCancellable *cancellable;
-  GError *error = NULL;
-  GFile *file = G_FILE (source_object);
-  GFile *unique_file = NULL;
-  GFileOutputStream *stream = NULL;
-  GTask *task = G_TASK (user_data);
-  PhotosUtilsFileCreateData *data;
-  gchar *filename = NULL;
-
-  cancellable = g_task_get_cancellable (task);
-  data = (PhotosUtilsFileCreateData *) g_task_get_task_data (task);
-
-  stream = g_file_create_finish (file, res, &error);
-  if (error != NULL)
-    {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
-        {
-          g_task_return_error (task, error);
-          goto out;
-        }
-
-      if (data->count == G_MAXUINT)
-        {
-          g_task_return_new_error (task, PHOTOS_ERROR, 0, "Exceeded number of copies of a file");
-          goto out;
-        }
-
-      data->count++;
-
-      filename = photos_utils_file_create_data_get_filename (data);
-      unique_file = g_file_get_child (data->dir, filename);
-
-      g_file_create_async (unique_file,
-                           data->flags,
-                           data->io_priority,
-                           cancellable,
-                           photos_utils_file_create_create,
-                           g_object_ref (task));
-
-      goto out;
-    }
-
-  g_task_return_pointer (task, g_object_ref (stream), g_object_unref);
-
- out:
-  g_free (filename);
-  g_clear_object (&stream);
-  g_clear_object (&unique_file);
-  g_object_unref (task);
-}
-
-
-void
-photos_utils_file_create_async (GFile *file,
-                                GFileCreateFlags flags,
-                                gint io_priority,
-                                GCancellable *cancellable,
-                                GAsyncReadyCallback callback,
-                                gpointer user_data)
-{
-  GTask *task;
-  PhotosUtilsFileCreateData *data;
-
-  task = g_task_new (file, cancellable, callback, user_data);
-  g_task_set_source_tag (task, photos_utils_file_create_async);
-
-  data = photos_utils_file_create_data_new (file, flags, io_priority);
-  g_task_set_task_data (task, data, (GDestroyNotify) photos_utils_file_create_data_free);
-
-  g_file_create_async (file,
-                       data->flags,
-                       data->io_priority,
-                       cancellable,
-                       photos_utils_file_create_create,
-                       g_object_ref (task));
-
-  g_object_unref (task);
-}
-
-
-GFileOutputStream *
-photos_utils_file_create_finish (GFile *file, GAsyncResult *res, GFile **out_unique_file, GError **error)
-{
-  GTask *task = G_TASK (res);
-  GFileOutputStream *ret_val = NULL;
-  PhotosUtilsFileCreateData *data;
-
-  g_return_val_if_fail (g_task_is_valid (res, file), NULL);
-  g_return_val_if_fail (g_task_get_source_tag (task) == photos_utils_file_create_async, NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-  data = (PhotosUtilsFileCreateData *) g_task_get_task_data (task);
-  g_return_val_if_fail (data != NULL, NULL);
-
-  ret_val = g_task_propagate_pointer (task, error);
-  if (ret_val == NULL)
-    goto out;
-
-  if (out_unique_file != NULL)
-    {
-      GFile *unique_file;
-      gchar *filename = NULL;
-
-      filename = photos_utils_file_create_data_get_filename (data);
-      unique_file = g_file_get_child (data->dir, filename);
-      *out_unique_file = unique_file;
-      g_free (filename);
-    }
-
- out:
-  return ret_val;
-}
-
-
-GQuark
-photos_utils_flash_off_quark (void)
-{
-  return g_quark_from_static_string ("http://www.tracker-project.org/temp/nmm#flash-off");
-}
-
-
-GQuark
-photos_utils_flash_on_quark (void)
-{
-  return g_quark_from_static_string ("http://www.tracker-project.org/temp/nmm#flash-on");
-}
-
-
 gchar *
 photos_utils_get_extension_from_mime_type (const gchar *mime_type)
 {
@@ -1318,18 +1104,48 @@ photos_utils_get_provider_name (PhotosBaseManager *src_mngr, PhotosBaseItem *ite
 }
 
 
-GtkBorder *
-photos_utils_get_thumbnail_frame_border (void)
+gchar *
+photos_utils_get_thumbnail_path_for_file (GFile *file)
 {
-  GtkBorder *slice;
+  gchar *path;
+  gchar *uri = NULL;
 
-  slice = gtk_border_new ();
-  slice->top = 3;
-  slice->right = 3;
-  slice->bottom = 6;
-  slice->left = 4;
+  uri = g_file_get_uri (file);
+  path = photos_utils_get_thumbnail_path_for_uri (uri);
 
-  return slice;
+  g_free (uri);
+  return path;
+}
+
+
+gchar *
+photos_utils_get_thumbnail_path_for_uri (const gchar *uri)
+{
+  const gchar *cache_dir;
+  gchar *filename = NULL;
+  gchar *md5 = NULL;
+  gchar *path;
+  gchar *thumbnails_subdir = NULL;
+  gint size;
+
+  md5 = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, -1);
+  filename = g_strconcat (md5, ".png", NULL);
+
+  cache_dir = g_get_user_cache_dir ();
+  size = photos_utils_get_icon_size ();
+  thumbnails_subdir = g_strdup_printf ("%d-%d", size, THUMBNAIL_GENERATION);
+
+  path = g_build_filename (cache_dir,
+                           PACKAGE_TARNAME,
+                           "thumbnails",
+                           thumbnails_subdir,
+                           filename,
+                           NULL);
+
+  g_free (filename);
+  g_free (md5);
+  g_free (thumbnails_subdir);
+  return path;
 }
 
 
@@ -1387,29 +1203,6 @@ photos_utils_list_box_header_func (GtkListBoxRow *row, GtkListBoxRow *before, gp
 }
 
 
-gboolean
-photos_utils_make_directory_with_parents (GFile *file, GCancellable *cancellable, GError **error)
-{
-  GError *local_error = NULL;
-  gboolean ret_val;
-
-  ret_val = g_file_make_directory_with_parents (file, cancellable, &local_error);
-  if (local_error != NULL)
-    {
-      if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_EXISTS))
-        {
-          g_clear_error (&local_error);
-          ret_val = TRUE;
-        }
-    }
-
-  if (local_error != NULL)
-    g_propagate_error (error, local_error);
-
-  return ret_val;
-}
-
-
 void
 photos_utils_object_list_free_full (GList *objects)
 {
@@ -1417,34 +1210,6 @@ photos_utils_object_list_free_full (GList *objects)
 }
 
 
-GQuark
-photos_utils_orientation_bottom_quark (void)
-{
-  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-bottom");
-}
-
-
-GQuark
-photos_utils_orientation_left_quark (void)
-{
-  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-left");
-}
-
-
-GQuark
-photos_utils_orientation_right_quark (void)
-{
-  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-right");
-}
-
-
-GQuark
-photos_utils_orientation_top_quark (void)
-{
-  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-top");
-}
-
-
 static void
 photos_utils_update_executed (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -1462,34 +1227,6 @@ photos_utils_update_executed (GObject *source_object, GAsyncResult *res, gpointe
 }
 
 
-void
-photos_utils_remove_children_from_node (GeglNode *node)
-{
-  GeglNode *input;
-  GeglNode *last;
-  GeglNode *output;
-  GeglOperation *operation;
-
-  operation = gegl_node_get_gegl_operation (node);
-  g_return_if_fail (operation == NULL);
-
-  input = gegl_node_get_input_proxy (node, "input");
-  output = gegl_node_get_output_proxy (node, "output");
-  last = gegl_node_get_producer (output, "input", NULL);
-
-  while (last != NULL && last != input)
-    {
-      GeglNode *last2;
-
-      last2 = gegl_node_get_producer (last, "input", NULL);
-      gegl_node_remove_child (node, last);
-      last = last2;
-    }
-
-  gegl_node_link (input, output);
-}
-
-
 void
 photos_utils_set_edited_name (const gchar *urn, const gchar *title)
 {
