@@ -30,7 +30,6 @@
 
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gegl.h>
 #include <gio/gio.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -44,14 +43,6 @@
 
 G_BEGIN_DECLS
 
-#define PHOTOS_ERROR (photos_utils_error_quark ())
-#define PHOTOS_FLASH_OFF (photos_utils_flash_off_quark ())
-#define PHOTOS_FLASH_ON (photos_utils_flash_on_quark ())
-#define PHOTOS_ORIENTATION_BOTTOM (photos_utils_orientation_bottom_quark ())
-#define PHOTOS_ORIENTATION_LEFT (photos_utils_orientation_left_quark ())
-#define PHOTOS_ORIENTATION_RIGHT (photos_utils_orientation_right_quark ())
-#define PHOTOS_ORIENTATION_TOP (photos_utils_orientation_top_quark ())
-
 #define PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME "photos-base-item"
 #define PHOTOS_SHARE_POINT_EXTENSION_POINT_NAME "photos-share-point"
 #define PHOTOS_SHARE_POINT_ONLINE_EXTENSION_POINT_NAME "photos-share-point-online"
@@ -62,33 +53,14 @@ G_BEGIN_DECLS
   "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#image-category-screenshot"
 #define PHOTOS_EXPORT_SUBPATH "Exports"
 
-gboolean         photos_utils_app_info_launch_uri         (GAppInfo *appinfo,
-                                                           const gchar *uri,
-                                                           GAppLaunchContext *launch_context,
-                                                           GError **error);
-
 void             photos_utils_border_pixbuf               (GdkPixbuf *pixbuf);
 
-void             photos_utils_buffer_zoom_async           (GeglBuffer *buffer,
-                                                           gdouble zoom,
-                                                           GCancellable *cancellable,
-                                                           GAsyncReadyCallback callback,
-                                                           gpointer user_data);
-
-GeglBuffer      *photos_utils_buffer_zoom_finish          (GeglBuffer *buffer, GAsyncResult *res, GError **error);
-
 GdkPixbuf       *photos_utils_center_pixbuf               (GdkPixbuf *pixbuf, gint size);
 
 gchar           *photos_utils_convert_path_to_uri         (const gchar *path);
 
-GeglBuffer      *photos_utils_create_buffer_from_node     (GeglNode *node);
-
 GIcon           *photos_utils_create_collection_icon      (gint base_size, GList *pixbufs);
 
-GeglNode        *photos_utils_create_orientation_node     (GeglNode *parent, GQuark orientation);
-
-GdkPixbuf       *photos_utils_create_pixbuf_from_node     (GeglNode *node);
-
 GdkPixbuf       *photos_utils_create_placeholder_icon_for_scale (const gchar *name, gint size, gint scale);
 
 GIcon           *photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale);
@@ -96,6 +68,10 @@ GIcon           *photos_utils_create_symbolic_icon_for_scale (const gchar *name,
 gboolean         photos_utils_create_thumbnail            (GFile *file,
                                                            const gchar *mime_type,
                                                            gint64 mtime,
+                                                           GQuark orientation,
+                                                           gint64 original_height,
+                                                           gint64 original_width,
+                                                           const gchar *pipeline_uri,
                                                            GCancellable *cancellable,
                                                            GError **error);
 
@@ -123,31 +99,35 @@ void             photos_utils_ensure_extension_points     (void);
 
 gboolean         photos_utils_equal_double                (gdouble a, gdouble b);
 
-GQuark           photos_utils_error_quark                 (void);
-
 gdouble          photos_utils_eval_radial_line            (gdouble crop_center_x,
                                                            gdouble crop_center_y,
                                                            gdouble corner_x,
                                                            gdouble corner_y,
                                                            gdouble event_x);
 
-void             photos_utils_file_create_async           (GFile *file,
-                                                           GFileCreateFlags flags,
+gboolean         photos_utils_file_copy_as_thumbnail      (GFile *source,
+                                                           GFile *destination,
+                                                           const gchar *original_uri,
+                                                           gint64 original_height,
+                                                           gint64 original_width,
+                                                           GCancellable *cancellable,
+                                                           GError **error);
+
+GFileInfo       *photos_utils_file_query_info             (GFile *file,
+                                                           const gchar *attributes,
+                                                           GFileQueryInfoFlags flags,
+                                                           GCancellable *cancellable,
+                                                           GError **error);
+
+void             photos_utils_file_query_info_async       (GFile *file,
+                                                           const gchar *attributes,
+                                                           GFileQueryInfoFlags flags,
                                                            gint io_priority,
                                                            GCancellable *cancellable,
                                                            GAsyncReadyCallback callback,
                                                            gpointer user_data);
 
-GFileOutputStream *photos_utils_file_create_finish        (GFile *file,
-                                                           GAsyncResult *res,
-                                                           GFile **out_unique_file,
-                                                           GError **error);
-
-gchar           *photos_utils_filename_strip_extension    (const gchar *filename_with_extension);
-
-GQuark           photos_utils_flash_off_quark             (void);
-
-GQuark           photos_utils_flash_on_quark              (void);
+GFileInfo       *photos_utils_file_query_info_finish      (GFile *file, GAsyncResult *res, GError **error);
 
 void             photos_utils_get_controller              (PhotosWindowMode mode,
                                                            PhotosOffsetController **out_offset_cntrlr,
@@ -164,7 +144,9 @@ char*            photos_utils_get_pixbuf_common_suffix    (GdkPixbufFormat *form
 
 const gchar     *photos_utils_get_provider_name           (PhotosBaseManager *src_mngr, PhotosBaseItem *item);
 
-GtkBorder       *photos_utils_get_thumbnail_frame_border  (void);
+gchar           *photos_utils_get_thumbnail_path_for_file (GFile *file);
+
+gchar           *photos_utils_get_thumbnail_path_for_uri  (const gchar *uri);
 
 GList           *photos_utils_get_urns_from_items         (GList *items);
 
@@ -174,20 +156,8 @@ void             photos_utils_list_box_header_func        (GtkListBoxRow *row,
                                                            GtkListBoxRow *before,
                                                            gpointer user_data);
 
-gboolean         photos_utils_make_directory_with_parents (GFile *file, GCancellable *cancellable, GError **error);
-
 void             photos_utils_object_list_free_full       (GList *objects);
 
-GQuark           photos_utils_orientation_bottom_quark    (void);
-
-GQuark           photos_utils_orientation_left_quark      (void);
-
-GQuark           photos_utils_orientation_right_quark     (void);
-
-GQuark           photos_utils_orientation_top_quark       (void);
-
-void             photos_utils_remove_children_from_node   (GeglNode *node);
-
 void             photos_utils_set_edited_name             (const gchar *urn, const gchar *title);
 
 void             photos_utils_set_favorite                (const gchar *urn, gboolean is_favorite);
